﻿using System;
using System.Collections.Generic;
using System.Data.Objects;
using System.Linq;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.Core.Domain.Repository;

namespace Palmmedia.MvcSampleApplication.Core.Database
{
    /// <summary>
    /// Implementation of the <see cref="Palmmedia.MvcSampleApplication.Core.Domain.Repository.IGenericRepository{T}"/> using the Entity Framework.
    /// </summary>
    /// <typeparam name="T">The <see cref="IObjectWithChangeTracker"/>.</typeparam>
    public class EFRepository<T> : IGenericRepository<T> where T : class, IObjectWithChangeTracker
    {
        /// <summary>
        /// The <see cref="ObjectContext"/> to use.
        /// </summary>
        private ObjectContext context;

        /// <summary>
        /// The <see cref="System.Data.Objects.IObjectSet{T}"/> to use.
        /// </summary>
        private IObjectSet<T> objectSet;

        /// <summary>
        /// Gets the <see cref="System.Data.Objects.ObjectContext"/> to use.
        /// </summary>
        private ObjectContext Context
        {
            get
            {
                if (this.context == null)
                {
                    this.context = GetCurrentUnitOfWork<EFUnitOfWork>().Context;
                }

                return this.context;
            }
        }

        /// <summary>
        /// Gets the <see cref="System.Data.Objects.IObjectSet{T}"/> to use.
        /// </summary>
        private IObjectSet<T> ObjectSet
        {
            get
            {
                if (this.objectSet == null)
                {
                    this.objectSet = this.Context.CreateObjectSet<T>();
                }

                return this.objectSet;
            }
        }

        /// <summary>
        /// Gets an <see cref="IQueryable{T}"/> to perform further operations.
        /// </summary>
        /// <returns>An <see cref="IQueryable{T}"/>.</returns>
        public IQueryable<T> GetQuery()
        {
            return this.ObjectSet;
        }

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements.
        /// </summary>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements.</returns>
        public IEnumerable<T> GetAll()
        {
            return this.GetQuery().ToList();
        }

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements matching the given condition.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements matching the given condition.</returns>
        public IEnumerable<T> Find(Func<T, bool> where)
        {
            return this.ObjectSet.Where<T>(where);
        }

        /// <summary>
        /// Gets a single instance matching the given condition.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>A single instance matching the given condition.</returns>
        public T Single(Func<T, bool> where)
        {
            return this.ObjectSet.Single<T>(where);
        }

        /// <summary>
        /// Gets a single instance matching the given condition or null if condition does not match.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>A single instance matching the given condition or <c>null</c> if condition does not match.</returns>
        public T SingleOrDefault(Func<T, bool> where)
        {
            return this.ObjectSet.SingleOrDefault<T>(where);
        }

        /// <summary>
        /// Loads the specified property of the given entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <param name="selector">The selector which contains the property to load.</param>
        public void LoadProperty(T entity, System.Linq.Expressions.Expression<Func<T, object>> selector)
        {
            this.Context.LoadProperty(entity, selector);
        }

        /// <summary>
        /// Deletes the given entity.
        /// </summary>
        /// <param name="entity">The entity to delete.</param>
        public void Delete(T entity)
        {
            this.ObjectSet.DeleteObject(entity);
        }

        /// <summary>
        /// Adds the given entity.
        /// </summary>
        /// <param name="entity">The entity to add.</param>
        public void Add(T entity)
        {
            this.ObjectSet.AddObject(entity);
        }

        /// <summary>
        /// Attaches the the given entity to the current context.
        /// </summary>
        /// <param name="entity">The entity to attach.</param>
        public void Attach(T entity)
        {
            this.ObjectSet.Attach(entity);
        }

        /// <summary>
        /// Applies the changes made to the given entity to the current context.
        /// </summary>
        /// <param name="entity">The entity whose changes should be applied.</param>
        public void ApplyChanges(T entity)
        {
            var objectSet = this.ObjectSet as ObjectSet<T>;
            objectSet.ApplyChanges(entity);
        }

        /// <summary>
        /// Gets the current <see cref="IUnitOfWork"/>.
        /// </summary>
        /// <typeparam name="TUnitOfWork">The Class implementing <see cref="IUnitOfWork"/>.</typeparam>
        /// <returns>The current <see cref="IUnitOfWork"/>.</returns>
        private static TUnitOfWork GetCurrentUnitOfWork<TUnitOfWork>() where TUnitOfWork : class, IUnitOfWork
        {
            return (TUnitOfWork)UnitOfWork.Current;
        }
    }
}
