﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities.Resources;

namespace Palmmedia.MvcSampleApplication.Core.Domain.Entities
{
    /// <summary>
    /// Extends validation based on <see cref="IDataErrorInfo"/>.
    /// </summary>
    public partial class User : IDataErrorInfo
    {
        /// <summary>
        /// String value representing the 'Name' property
        /// </summary>
        private const string NameProperty = "Name";

        /// <summary>
        /// String value representing the 'Password' property
        /// </summary>
        private const string PasswordProperty = "Password";

        /// <summary>
        /// Gets an array containing all properties to validate.
        /// </summary>
        private static readonly string[] ValidatedProperties = 
        { 
            NameProperty, 
            PasswordProperty
        };

        /// <summary>
        /// Initializes a new instance of the <see cref="User"/> class.
        /// </summary>
        public User()
        {
            this.Id = Guid.NewGuid();
            this.Created = DateTime.Now;
        }

        /// <summary>
        /// Gets an error message indicating what is wrong with this object.
        /// </summary>
        /// <value></value>
        /// <returns>An error message indicating what is wrong with this object. The default is an empty string ("").</returns>
        public string Error
        {
            get { return string.Empty; }
        }

        /// <summary>
        /// Gets a value indicating whether this object has validation errors.
        /// </summary>
        public bool IsValid
        {
            get
            {
                foreach (string property in ValidatedProperties)
                {
                    if (this[property] != null)
                    {
                        return false;
                    }
                }

                return true;
            }
        }

        /// <summary>
        /// Gets the error of the specified column name.
        /// </summary>
        /// <param name="columnName">The name of the property</param>
        public string this[string columnName]
        {
            get 
            {
                if (Array.IndexOf(ValidatedProperties, columnName) < 0)
                {
                    return null;
                }

                string error = null;

                switch (columnName)
                {
                    case NameProperty:
                        error = this.ValidateName();
                        break;

                    case PasswordProperty:
                        error = this.ValidatePassword();
                        break;

                    default:
                        Debug.Fail("Unexpected property being validated on User: " + columnName);
                        break;
                }

                return error;
            }
        }
        
        /// <summary>
        /// Validates the 'Name' property
        /// </summary>
        /// <returns>The error or null if none exists.</returns>
        private string ValidateName()
        {
            if (string.IsNullOrEmpty(this.Name))
            {
                return UserResources.Username;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Validated the 'Password' property
        /// </summary>
        /// <returns>The error or null if none exists.</returns>
        private string ValidatePassword()
        {
            if (string.IsNullOrEmpty(this.Password))
            {
                return UserResources.Password;
            }
            else
            {
                return null;
            }
        }
    }
}
