﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;

namespace Palmmedia.MvcSampleApplication.Core.Domain.Repository
{
    /// <summary>
    /// Interface for generic repository implementations.
    /// </summary>
    /// <typeparam name="T">Class instance of <see cref="IObjectWithChangeTracker"/>.</typeparam>
    public interface IGenericRepository<T> where T : class, IObjectWithChangeTracker
    {
        /// <summary>
        /// Gets an <see cref="IQueryable{T}"/> to perform further operations.
        /// </summary>
        /// <returns>An <see cref="IQueryable{T}"/>.</returns>
        IQueryable<T> GetQuery();

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements.
        /// </summary>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements.</returns>
        IEnumerable<T> GetAll();

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements matching the given condition.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements matching the given condition.</returns>
        IEnumerable<T> Find(Func<T, bool> where);

        /// <summary>
        /// Gets a single instance matching the given condition.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>A single instance matching the given condition.</returns>
        T Single(Func<T, bool> where);

        /// <summary>
        /// Gets a single instance matching the given condition or null if condition does not match.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>A single instance matching the given condition or <c>null</c> if condition does not match.</returns>
        T SingleOrDefault(Func<T, bool> where);

        /// <summary>
        /// Loads the specified property of the given entity.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <param name="selector">The selector which contains the property to load.</param>
        void LoadProperty(T entity, Expression<Func<T, object>> selector);

        /// <summary>
        /// Deletes the given entity.
        /// </summary>
        /// <param name="entity">The entity to delete.</param>
        void Delete(T entity);

        /// <summary>
        /// Adds the given entity.
        /// </summary>
        /// <param name="entity">The entity to add.</param>
        void Add(T entity);

        /// <summary>
        /// Attaches the the given entity to the current context.
        /// </summary>
        /// <param name="entity">The entity to attach.</param>
        void Attach(T entity);

        /// <summary>
        /// Applies the changes made to the given entity to the current context.
        /// </summary>
        /// <param name="entity">The entity whose changes should be applied.</param>
        void ApplyChanges(T entity);
    }
}
