﻿using System;
using System.Collections;
using System.Threading;
using System.Web;
using log4net;
using Palmmedia.Common.Ioc;

namespace Palmmedia.MvcSampleApplication.Core.Domain.Repository
{
    /// <summary>
    /// Manages the currently used instances of <see cref="IUnitOfWork"/>.
    /// If used in <see cref="System.Web.HttpContext"/> one instance of <see cref="IUnitOfWork"/> is used during one request.
    /// Otherwise one instance of <see cref="IUnitOfWork"/> is used per <see cref="System.Threading.Thread"/>
    /// </summary>
    public static class UnitOfWork
    {
        /// <summary>
        /// Logger instance.
        /// </summary>
        private static readonly ILog logger = LogManager.GetLogger(typeof(UnitOfWork));

        /// <summary>
        /// Key used to access instance of <see cref="IUnitOfWork"/> in case of beeing in <see cref="System.Web.HttpContext"/>.
        /// </summary>
        private const string HTTPCONTEXTKEY = "Palmmedia.MvcSampleApplication.Core.Domain.Repository.HttpContext.Key";

        /// <summary>
        /// Contains the instance of <see cref="IUnitOfWork"/> belonging to a thread.
        /// </summary>
        private static readonly Hashtable threads = new Hashtable();

        /// <summary>
        /// Gets the instance of <see cref="IUnitOfWork"/> depending on the current context.
        /// </summary>
        public static IUnitOfWork Current
        {
            get
            {
                var unitOfWork = GetUnitOfWork();
                if (unitOfWork == null)
                {
                    var unitOfWorkFactory = IocHelper.Resolve<IUnitOfWorkFactory>();
                    unitOfWork = unitOfWorkFactory.Create();
                    SaveUnitOfWork(unitOfWork);
                }

                return unitOfWork;
            }
        }

        /// <summary>
        /// Resets the current <see cref="IUnitOfWork"/>.
        /// </summary>
        public static void Reset()
        {
            logger.Debug("Resetting UnitOfWork of Thread: " + Thread.CurrentThread.Name);
            SaveUnitOfWork(null);
        }

        /// <summary>
        /// Saves the made changes.
        /// </summary>
        public static void Commit()
        {
            IUnitOfWork unitOfWork = GetUnitOfWork();
            if (unitOfWork != null)
            {
                logger.Debug("Commiting UnitOfWork of Thread: " + Thread.CurrentThread.Name);
                unitOfWork.Commit();
            }
        }

        /// <summary>
        /// Returns the cached instance of <see cref="IUnitOfWork"/> depending on the current context.
        /// Returns <c>null</c> if instance does not exist yet.
        /// </summary>
        /// <returns>The instance of <see cref="IUnitOfWork"/>.</returns>
        private static IUnitOfWork GetUnitOfWork()
        {
            if (HttpContext.Current != null)
            {
                if (HttpContext.Current.Items.Contains(HTTPCONTEXTKEY))
                {
                    logger.Debug("Retrieving UnitOfWork from HttpContext");
                    return (IUnitOfWork)HttpContext.Current.Items[HTTPCONTEXTKEY];
                }

                return null;
            }
            else
            {
                var thread = Thread.CurrentThread;
                if (string.IsNullOrEmpty(thread.Name))
                {
                    thread.Name = Guid.NewGuid().ToString();
                    return null;
                }
                else
                {
                    lock (threads.SyncRoot)
                    {
                        logger.Debug("Retrieving UnitOfWork of Thread: " + Thread.CurrentThread.Name);
                        return (IUnitOfWork)threads[Thread.CurrentThread.Name];
                    }
                }
            }
        }

        /// <summary>
        /// Saves the given instance of <see cref="IUnitOfWork"/> depending on the current context.
        /// </summary>
        /// <param name="unitOfWork">The instance of <see cref="IUnitOfWork"/>.</param>
        private static void SaveUnitOfWork(IUnitOfWork unitOfWork)
        {
            if (HttpContext.Current != null)
            {
                logger.Debug("Saving UnitOfWork in HttpContext");
                HttpContext.Current.Items[HTTPCONTEXTKEY] = unitOfWork;
            }
            else
            {
                lock (threads.SyncRoot)
                {
                    logger.Debug("Saving UnitOfWork of Thread: " + Thread.CurrentThread.Name);
                    threads[Thread.CurrentThread.Name] = unitOfWork;
                }
            }
        }
    }
}
