using System.Globalization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Threading;

namespace Palmmedia.MvcSampleApplication.Service.WcfFacade.I18n
{
    /// <summary>
    /// Using this <see cref="I18nMessageInspector"/> allows to run client and server threads in the same culture for localization purposes.
    /// The <see cref="I18nMessageInspector"/> implements <see cref="IClientMessageInspector"/> to pass a string containing the TwoLetterISOLanguageName of the current thread culture to the server.
    /// It also implements <see cref="IDispatchMessageInspector"/> which reads the TwoLetterISOLanguageName and applies the given TwoLetterISOLanguageName to the current thread on the server side.
    /// </summary>
    public class I18nMessageInspector : IClientMessageInspector, IDispatchMessageInspector
    {
        /// <summary>
        /// The name of the header element.
        /// </summary>
        private const string Locale = "Locale";

        /// <summary>
        /// The namespace of the header element.
        /// </summary>
        private const string NamespaceURI = "http://www.palmmedia.de/mvcsampleapplication/i18n";

        #region IClientMessageInspector Members
        /// <summary>
        /// Enables inspection or modification of a message after a reply message is received but prior to passing it back to the client application.
        /// </summary>
        /// <param name="reply">The message to be transformed into types and handed back to the client application.</param>
        /// <param name="correlationState">Correlation state data.</param>
        public void AfterReceiveReply(ref Message reply, object correlationState)
        {
        }

        /// <summary>
        /// Enables inspection or modification of a message before a request message is sent to a service.
        /// </summary>
        /// <param name="request">The message to be sent to the service.</param>
        /// <param name="channel">The  client object channel.</param>
        /// <returns>
        /// <c>null</c> is returned.
        /// </returns>
        public object BeforeSendRequest(ref Message request, IClientChannel channel)
        {
            var header = MessageHeader.CreateHeader(Locale, NamespaceURI, Thread.CurrentThread.CurrentCulture.TwoLetterISOLanguageName);
            request.Headers.Add(header);
            return null;
        }
        #endregion

        #region IDispatchMessageInspector Members
        /// <summary>
        /// Called after an inbound message has been received but before the message is dispatched to the intended operation.
        /// </summary>
        /// <param name="request">The request message.</param>
        /// <param name="channel">The incoming channel.</param>
        /// <param name="instanceContext">The current service instance.</param>
        /// <returns>
        /// <c>null</c> is returned.
        /// </returns>
        public object AfterReceiveRequest(ref Message request, IClientChannel channel, InstanceContext instanceContext)
        {
            var locale = string.Empty;

            try
            {
                locale = request.Headers.GetHeader<string>(Locale, NamespaceURI);
            }
            catch (MessageHeaderException)
            { 
            }

            var cultureInfo = CultureInfo.InvariantCulture;

            if (!string.IsNullOrEmpty(locale) && locale.Length == 2 && !locale.Equals("iv", System.StringComparison.OrdinalIgnoreCase))
            {
                cultureInfo = CultureInfo.CreateSpecificCulture(locale);
            }

            Thread.CurrentThread.CurrentCulture = cultureInfo;
            Thread.CurrentThread.CurrentUICulture = cultureInfo;

            return null;
        }

        /// <summary>
        /// Called after the operation has returned but before the reply message is sent.
        /// </summary>
        /// <param name="reply">The reply message. This value is null if the operation is one way.</param>
        /// <param name="correlationState">The correlation object returned from the <see cref="M:System.ServiceModel.Dispatcher.IDispatchMessageInspector.AfterReceiveRequest(System.ServiceModel.Channels.Message@,System.ServiceModel.IClientChannel,System.ServiceModel.InstanceContext)"/> method.</param>
        public void BeforeSendReply(ref Message reply, object correlationState)
        {
        }
        #endregion
    }
}
