﻿using System;
using System.Collections.Generic;
using System.Linq;
using log4net;
using Palmmedia.Common.Ioc;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.Core.Domain.Repository;

namespace Palmmedia.MvcSampleApplication.Service.WcfFacade
{
    /// <summary>
    /// Implementation of <see cref="IServiceFacade"/>.
    /// </summary>
    [ResetUnitOfWork()]
    public class ServiceFacade : IServiceFacade
    {
        /// <summary>
        /// Logger instance.
        /// </summary>
        private static readonly ILog logger = LogManager.GetLogger(typeof(ServiceFacade));

        /// <summary>
        /// Adds the given <see cref="User"/>.
        /// </summary>
        /// <param name="user">The <see cref="User"/> to add.</param>
        public void AddUser(User user)
        {
            if (user == null)
            {
                throw new ArgumentNullException("user");
            }

            logger.Debug("Adding user (Id: " + user.Id + ").");

            var repository = IocHelper.Resolve<IGenericRepository<User>>();

            repository.Add(user);
            UnitOfWork.Commit();
        }

        /// <summary>
        /// Gets the <see cref="User"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id of the <see cref="User"/>.</param>
        /// <returns>The <see cref="User"/> with the given Id.</returns>
        public User GetUser(Guid id)
        {
            logger.Debug("Retrieving user (Id: " + id + ").");

            var repository = IocHelper.Resolve<IGenericRepository<User>>();

            var user = repository.SingleOrDefault(u => u.Id.Equals(id));
            repository.LoadProperty(user, u => u.Roles);
            repository.LoadProperty(user, u => u.GuestbookEntries);
            return user;
        }

        /// <summary>
        /// Gets the <see cref="User"/> with the given name.
        /// </summary>
        /// <param name="name">The name of the <see cref="User"/>.</param>
        /// <returns>The <see cref="User"/> with the given name.</returns>
        public User GetUser(string name)
        {
            logger.Debug("Retrieving user (Name: " + name + ").");

            if (name == null)
            {
                throw new ArgumentNullException("name");
            }

            var repository = IocHelper.Resolve<IGenericRepository<User>>();

            var user = repository.SingleOrDefault(u => u.Name.Equals(name));
            repository.LoadProperty(user, u => u.Roles);
            return user;
        }

        /// <summary>
        /// Determinates whether an <see cref="User"/> with the given name and password exists.
        /// </summary>
        /// <param name="name">The name of the <see cref="User"/>.</param>
        /// <param name="password">The password of th <see cref="User"/>.</param>
        /// <returns>True if the <see cref="User"/> exists, otherwise false.</returns>
        public bool UserExists(string name, string password)
        {
            logger.Debug("Checking user existince (Name: " + name + ").");

            if (name == null)
            {
                throw new ArgumentNullException("name");
            }

            if (password == null)
            {
                throw new ArgumentNullException("password");
            }

            var repository = IocHelper.Resolve<IGenericRepository<User>>();

            return repository.GetQuery().Where(u => u.Name.Equals(name) && u.Password.Equals(password)).Count() == 1;
        }

        /// <summary>
        /// Updates the given <see cref="User"/>.
        /// </summary>
        /// <param name="user">The <see cref="User"/> to update.</param>
        public void UpdateUser(User user)
        {
            if (user == null)
            {
                throw new ArgumentNullException("user");
            }

            logger.Debug("Updating user (Id: " + user.Id + ").");

            var repository = IocHelper.Resolve<IGenericRepository<User>>();

            repository.ApplyChanges(user);
            UnitOfWork.Commit();
        }

        /// <summary>
        /// Gets all <see cref="User">Users</see>.
        /// </summary>
        /// <returns>All <see cref="User">Users</see>.</returns>
        public IEnumerable<User> GetUsers()
        {
            logger.Debug("Retrieving all users.");

            var repository = IocHelper.Resolve<IGenericRepository<User>>();

            return repository.GetAll();
        }

        /// <summary>
        /// Adds the given <see cref="GuestbookEntry"/>.
        /// </summary>
        /// <param name="guestbookEntry">The <see cref="GuestbookEntry"/> to add.</param>
        public void AddGuestbookEntry(GuestbookEntry guestbookEntry)
        {
            if (guestbookEntry == null)
            {
                throw new ArgumentNullException("guestbookEntry");
            }

            logger.Debug("Adding guestbook entry (Id: " + guestbookEntry.Id + ").");

            var repository = IocHelper.Resolve<IGenericRepository<GuestbookEntry>>();

            repository.ApplyChanges(guestbookEntry);
            UnitOfWork.Commit();
        }

        /// <summary>
        /// Gets all <see cref="GuestbookEntry">GuestbookEntries</see>.
        /// </summary>
        /// <returns>All <see cref="GuestbookEntry">GuestbookEntries</see>.</returns>
        public IEnumerable<GuestbookEntry> GetGuestbookEntries()
        {
            logger.Debug("Retrieving all guestbook entries.");

            var repository = IocHelper.Resolve<IGenericRepository<GuestbookEntry>>();

            var entries = repository.GetQuery().OrderByDescending(e => e.Created);

            foreach (var entry in entries)
            {
                repository.LoadProperty(entry, e => e.User);
            }

            return entries;
        }

        /// <summary>
        /// Gets the <see cref="GuestbookEntry"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id of the <see cref="GuestbookEntry"/>.</param>
        /// <returns>The <see cref="GuestbookEntry"/>.</returns>
        public GuestbookEntry GetGuestbookEntry(Guid id)
        {
            logger.Debug("Retrieving guestbook entry (Id: " + id + ").");

            var repository = IocHelper.Resolve<IGenericRepository<GuestbookEntry>>();

            var entry = repository.SingleOrDefault(u => u.Id.Equals(id));
            repository.LoadProperty(entry, e => e.User);
            return entry;
        }

        /// <summary>
        /// Gets all <see cref="GuestbookEntry">GuestbookEntries</see> of the the <see cref="User"/> with the given Id.
        /// </summary>
        /// <param name="userId">The Id of the <see cref="User"/>.</param>
        /// <returns>All <see cref="GuestbookEntry">GuestbookEntries</see> of the the <see cref="User"/>.</returns>
        public IEnumerable<GuestbookEntry> GetGuestbookEntriesByUser(Guid userId)
        {
            logger.Debug("Retrieving all guestbook entries of user (Id: " + userId + ").");

            var repository = IocHelper.Resolve<IGenericRepository<GuestbookEntry>>();

            var entries = repository.GetQuery().Where(entry => entry.User.Id.Equals(userId)).OrderByDescending(e => e.Created);

            foreach (var entry in entries)
            {
                repository.LoadProperty(entry, e => e.User);
            }

            return entries;
        }

        /// <summary>
        /// Updates the given <see cref="GuestbookEntry"/>.
        /// </summary>
        /// <param name="guestbookEntry">The <see cref="GuestbookEntry"/> to update.</param>
        public void UpdateGuestbookEntry(GuestbookEntry guestbookEntry)
        {
            if (guestbookEntry == null)
            {
                throw new ArgumentNullException("guestbookEntry");
            }

            logger.Debug("Updating guestbook entry (Id: " + guestbookEntry.Id + ").");

            var repository = IocHelper.Resolve<IGenericRepository<GuestbookEntry>>();

            repository.ApplyChanges(guestbookEntry);
            UnitOfWork.Commit();
        }

        /// <summary>
        /// Deletes the <see cref="GuestbookEntry"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id of the <see cref="GuestbookEntry"/>.</param>
        public void DeleteGuestbookEntry(Guid id)
        {
            var repository = IocHelper.Resolve<IGenericRepository<GuestbookEntry>>();

            logger.Debug("Deleting guestbook entry (Id: " + id + ").");

            var guestbookEntry = repository.SingleOrDefault(u => u.Id.Equals(id));
            repository.Delete(guestbookEntry);
            UnitOfWork.Commit();
        }
    }
}
