﻿using System;
using System.Web.Mvc;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.UI.AspMvc.Authorization;

namespace Palmmedia.MvcSampleApplication.UI.AspMvc.Controllers
{
    /// <summary>
    /// Controller for guestbook.
    /// </summary>
    [Palmmedia.Common.Net.Mvc.HandleError]
    [Authorize]
    public partial class GuestbookController : ControllerBase
    {
        /// <summary>
        /// Shows the guestbook overview.
        /// </summary>
        /// <returns>The guestbook overview.</returns>
        public virtual ActionResult Index()
        {
            var entries = this.ServiceFacade.GetGuestbookEntries();

            return View(entries);
        }

        /// <summary>
        /// Adds a new guestbook entry.
        /// </summary>
        /// <param name="comment">The comment.</param>
        /// <returns>The guestbook overview.</returns>
        [ValidateInput(false)] 
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Index(string comment)
        {
            var entry = new GuestbookEntry();
            try
            {
                this.UpdateModel<GuestbookEntry>(entry, new string[] { "Comment" });
            }
            catch (InvalidOperationException)
            {
                return View();
            }

            entry.User = this.GetUser();
            entry.User.GuestbookEntries.Add(entry);

            this.ServiceFacade.AddGuestbookEntry(entry);
           
            return this.RedirectToAction(MVC.Guestbook.Index());
        } 

        /// <summary>
        /// Shows a view, where a guestbook entry can be edited.
        /// </summary>
        /// <param name="id">The Id of the guestbook entry.</param>
        /// <returns>View showing a single guestbook entry.</returns>
        [Authorize(Roles = "Admin")]
        public virtual ActionResult Edit(Guid? id)
        {
            if (id == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            Palmmedia.MvcSampleApplication.Core.Domain.Entities.GuestbookEntry entry = this.ServiceFacade.GetGuestbookEntry(id.Value);
    
            if (entry == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            return View(entry);
        }

        /// <summary>
        /// Saves the edited guestbook entry.
        /// </summary>
        /// <param name="id">The Id of the guestbook entry.</param>
        /// <param name="comment">The new comment.</param>
        /// <returns>The guestbook overview.</returns>
        [ValidateInput(false)] 
        [Authorize(Roles = "Admin")]
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Edit(Guid? id, string comment)
        {
            if (id == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            GuestbookEntry entry = this.ServiceFacade.GetGuestbookEntry(id.Value);

            if (entry == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            try
            {
                this.UpdateModel<GuestbookEntry>(entry, new string[] { "Comment" });
            }
            catch (InvalidOperationException)
            {
                return View();
            }

            this.ServiceFacade.UpdateGuestbookEntry(entry);

            return this.RedirectToAction(MVC.Guestbook.Index());
        }        

        /// <summary>
        /// Shows a confirmation view to delete a guestbook entry.
        /// </summary>
        /// <param name="id">The Id of the guestbook entry.</param>
        /// <returns>The confirmation view.</returns>
        [Authorize(Roles = "Admin")]
        public virtual ActionResult Delete(Guid? id)
        {
            if (id == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            Palmmedia.MvcSampleApplication.Core.Domain.Entities.GuestbookEntry entry = this.ServiceFacade.GetGuestbookEntry(id.Value);
    

            if (entry == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            return View(entry);
        }

        /// <summary>
        /// Deletes the guestbookentry with the given id.
        /// </summary>
        /// <param name="id">The Id of the guestbook entry.</param>
        /// <param name="confirmButton">The confirm button.</param>
        /// <returns>The guestbook overview.</returns>
        [ValidateInput(false)] 
        [Authorize(Roles = "Admin")]
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Delete(Guid? id, string confirmButton)
        {
            if (id == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            this.ServiceFacade.DeleteGuestbookEntry(id.Value);

            return this.RedirectToAction(MVC.Guestbook.Index());           
        }

        /// <summary>
        /// Shows the guestbook entries of an user.
        /// </summary>
        /// <param name="id">The Id of an user.</param>
        /// <returns>View showing guestbook entries of an user.</returns>
        public virtual ActionResult EntriesOfUser(Guid? id)
        {
            if (id == null)
            {
                return View(MVC.Guestbook.Views.NotFoundUser);
            }

            Palmmedia.MvcSampleApplication.Core.Domain.Entities.User user = this.ServiceFacade.GetUser(id.Value);
         
            if (user == null)
            {
                return View(MVC.Guestbook.Views.NotFound);
            }

            return View(user);
        }
    }
}
