﻿using System;
using System.Web.Mvc;
using System.Web.Security;
using Palmmedia.Common.Net.Mvc.Localization;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;

namespace Palmmedia.MvcSampleApplication.UI.AspMvc.Controllers
{
    /// <summary>
    /// Controller for login.
    /// </summary>
    [Palmmedia.Common.Net.Mvc.HandleError]
    public partial class LoginController : ControllerBase
    {
        /// <summary>
        /// Shows the login view.
        /// </summary>
        /// <returns>The login view.</returns>
        public virtual ActionResult Index()
        {
            return View();            
        }

        /// <summary>
        /// Performs a login.
        /// </summary>
        /// <param name="name">The username.</param>
        /// <param name="password">The password.</param>
        /// <param name="rememberMe">Determinates whether a cookie should be set.</param>
        /// <param name="returnUrl">The url to redirect to (after successful login).</param>
        /// <returns>The desired view.</returns>
        [ValidateInput(false)]
        [Palmmedia.Common.Net.Mvc.ReferrerAuthorize()]
        [ValidateAntiForgeryToken()]
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Index(string name, string password, bool rememberMe, string returnUrl)
        {
            var user = new User();

            try
            {
                this.UpdateModel<User>(user, new string[] { "Name", "Password" });
            }
            catch (InvalidOperationException)
            {
                return View();
            }

            if (!Membership.Provider.ValidateUser(name, password))
            {
                ModelState.AddModelError("Login", this.Resource("LoginController, Login"));
                return View();
            }

            FormsAuthentication.SetAuthCookie(name, rememberMe);

            if (!string.IsNullOrEmpty(returnUrl))
            {
                return Redirect(returnUrl);
            }
            else
            {
                return this.RedirectToAction(MVC.Home.Index());
            }
        }

        /// <summary>
        /// Shows the logout view.
        /// </summary>
        /// <returns>The logout view.</returns>
        public virtual ActionResult LogOff()
        {
            if (!this.Request.IsAuthenticated)
            {
                return this.RedirectToAction(MVC.Login.Index());
            }

            return View();
        }

        /// <summary>
        /// Performs the logout.
        /// </summary>
        /// <param name="values">The FormCollection.</param>
        /// <returns>The login view.</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult LogOff(FormCollection values)
        {
            FormsAuthentication.SignOut();

            return this.RedirectToAction(MVC.Login.Index());
        }

        /// <summary>
        /// Shows the register view.
        /// </summary>
        /// <returns>The register view.</returns>
        public virtual ActionResult Register()
        {
            return View();
        }

        /// <summary>
        /// Registers a user.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="password">The password.</param>
        /// <param name="confirmPassword">The confirm password.</param>
        /// <returns>The login view.</returns>
        [ValidateInput(false)] 
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Register(string name, string password, string confirmPassword)
        {
            var user = new User();

            try
            {
                this.UpdateModel<User>(user, new string[] { "Name", "Password" });
                if (!this.ValidatePassword(password, confirmPassword))
                {
                    return View();
                }
            }
            catch (InvalidOperationException)
            {
                return View();
            }

            user.Roles.Add(new Role { User = user, Name = Palmmedia.MvcSampleApplication.Core.Domain.Entities.Roles.Editor.ToString() });

            this.ServiceFacade.AddUser(user);

            return this.RedirectToAction(MVC.Login.Index());
        }

        /// <summary>
        /// Validates the password.
        /// </summary>
        /// <param name="password">The password.</param>
        /// <param name="confirmPassword">The confirm password.</param>
        /// <returns>True if passwords are equal, otherwise false.</returns>
        private bool ValidatePassword(string password, string confirmPassword)
        {
            if (string.IsNullOrEmpty(confirmPassword))
            {
                ModelState.AddModelError("ConfirmPassword", this.Resource("LoginController, ConfirmPassword"));
            }

            if (password != null && !password.Equals(confirmPassword))
            {
                ModelState.AddModelError("Password", this.Resource("LoginController, PasswordMismatch"));
            }

            return ModelState.IsValid;
        }
    }
}
