﻿using System;
using System.Windows.Input;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.UI.Wpf.Common;

namespace Palmmedia.MvcSampleApplication.UI.Wpf.ViewModels
{
    /// <summary>
    /// ViewModel for <see cref="Palmmedia.MvcSampleApplication.UI.Wpf.Views.LoginView"/>.
    /// </summary>
    internal class LoginViewModel : ViewModelBase
    {
        /// <summary>
        /// The user.
        /// </summary>
        private User user;

        /// <summary>
        /// The login error.
        /// </summary>
        private string loginError;

        /// <summary>
        /// Executed when a login should be performed. 
        /// </summary>
        private ICommand loginCommand;

        /// <summary>
        /// Initializes a new instance of the <see cref="LoginViewModel"/> class.
        /// </summary>
        public LoginViewModel()
        {
            this.User = new User();
        }

        /// <summary>
        /// Raised when a login was performed successfully.
        /// </summary>
        public event EventHandler<EventArgs> LoginSuccessful;

        /// <summary>
        /// Gets or sets the user.
        /// </summary>
        public User User
        {
            get
            {
                return this.user;
            }

            set
            {
                this.user = value;
                OnPropertyChanged("User");
            }
        }

        /// <summary>
        /// Gets or sets the login error.
        /// </summary>
        public string LoginError
        {
            get
            {
                return this.loginError;
            }

            set
            {
                this.loginError = value;
                OnPropertyChanged("LoginError");
            }
        }

        /// <summary>
        /// Gets the command which is executed when a login should be performed.
        /// </summary>
        public ICommand LoginCommand
        {
            get
            {
                if (this.loginCommand == null)
                {
                    this.loginCommand = new RelayCommand(param => this.Login(), param => this.CanLogin);
                }

                return this.loginCommand;
            }
        }
        
        /// <summary>
        /// Gets a value indicating whether a login can be performed.
        /// </summary>
        private bool CanLogin
        {
            get { return this.User.IsValid; }
        }

        /// <summary>
        /// Raises the <see cref="E:LoginSuccessful"/> event.
        /// </summary>
        /// <param name="args">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected virtual void OnLoginSuccessful(EventArgs args)
        {
            if (this.LoginSuccessful != null)
            {
                this.LoginSuccessful(this, args);
            }
        }

        /// <summary>
        /// Logs in an user.
        /// </summary>
        private void Login()
        {
            var accepted = this.ServiceFacade.UserExists(this.User.Name, this.User.Password);

            if (accepted)
            {
                CurrentUser = this.ServiceFacade.GetUserByName(this.User.Name);
                this.OnLoginSuccessful(EventArgs.Empty);
            }
            else
            {
                this.LoginError = Palmmedia.MvcSampleApplication.UI.Wpf.Resources.Strings.Login_Failed;
            }
        }
    }
}
