﻿using System;
using System.ComponentModel;
using Palmmedia.Common.Ioc;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.Service.Proxy.ServiceReference;

namespace Palmmedia.MvcSampleApplication.UI.Wpf.ViewModels
{
    /// <summary>
    /// Base class for all ViewModel classes in the application.
    /// It provides support for property change notifications 
    /// and has a DisplayName property. This class is abstract.
    /// </summary>
    internal abstract class ViewModelBase : INotifyPropertyChanged, IDisposable
    {
        /// <summary>
        /// Raised when a property on this object has a new value.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets the currently logged in <see cref="User"/>.
        /// </summary>
        protected static User CurrentUser { get; set; }

        /// <summary>
        /// Gets the <see cref="IServiceFacade"/>.
        /// </summary>
        protected IServiceFacade ServiceFacade
        {
            get
            {
                return IocHelper.Resolve<IServiceFacade>();
            }
        }

        /// <summary>
        /// Invoked when this object is being removed from the application
        /// and will be subject to garbage collection.
        /// </summary>
        public void Dispose()
        {
            this.OnDispose();
        }

        /// <summary>
        /// Child classes can override this method to perform 
        /// clean-up logic, such as removing event handlers.
        /// </summary>
        protected virtual void OnDispose()
        {
        }

        /// <summary>
        /// Raises this object's PropertyChanged event.
        /// </summary>
        /// <param name="propertyName">The property that has a new value.</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}